import pandas as pd
import geopandas as gpd
import json

from sqlalchemy import text

class GeolevelDataType:
    # Constructor
    def __init__(
            self,
            geolevel_data_type:int,
            name:str,
            properties:dict,
            geolevel_id:int,
            description:str,
            org:'Org' # type: ignore
    ):
        self._geoleveldata_type_id = geolevel_data_type
        self._name = name
        self._properties = properties
        self._geolevel_id = geolevel_id
        self._description = description
        self._org = org

    # Getters
    @property
    def geoleveldata_type_id(self): return self._geoleveldata_type_id
    @property
    def name(self): return self._name
    @property
    def properties(self): return self._properties
    @property
    def geolevel_id(self): return self._geolevel_id
    @property
    def description(self): return self._description
    @property
    def org(self): return self._org

    # Public Methods
    def getGeolevel(self):
        return self._org.getGeolevelById(self._geolevel_id)
    
    def getDataset(self)->pd.DataFrame:
        query = text(f"SELECT * FROM ggo_geoleveldata WHERE geoleveldata_type_id = {self._geoleveldata_type_id}")
        return pd.read_sql(query, self._org.engine)
    
    def getGeoDataset(self)->gpd.GeoDataFrame:
        geolevel = self.getGeolevel()
        query = text(f"""
            SELECT
                g.*,
                geo.{geolevel.geom_field} AS geometry
            FROM ggo_geoleveldata g
            JOIN {geolevel.table_name} AS geo ON geo.{geolevel.geounit_code} = g.geounit_code
            WHERE
                geoleveldata_type_id = {self._geoleveldata_type_id}
        """)
        return gpd.read_postgis(query, self._org.engine, geom_col='geometry')
    
    def add_to_model(self) -> 'GeolevelDataType':
        # add to database
        query = f"""
            INSERT INTO ggo_geoleveldata_type (
                name,
                description,
                properties,
                geolevel_id
            ) VALUES (
                '{self._name.replace("'", "''")}',
                '{self._description.replace("'", "''")}',
                '{json.dumps(self._properties).replace("'", "''")}',
                {self._geolevel_id}
            ) RETURNING geoleveldata_type_id
        """
        geolevel_data_type_id = self._org.query(query)[0][0]
        self._geoleveldata_type_id = geolevel_data_type_id
        # return
        return self



import pandas as pd
import geopandas as gpd
from shapely.geometry import shape, mapping
from shapely.geometry.base import BaseGeometry
import json
from sqlalchemy import text

from galigeopy.model.poi import Poi
from galigeopy.model.zone_geounit import ZoneGeounit

class Zone:
    # Constructor
    def __init__(
        self,
        zone_id:int,
        properties:dict,
        geolevel_id:int,
        zone_type_id:int,
        poi_id:int,
        parent_zone_id:int,
        geometry:BaseGeometry,
        org:'Org' # type: ignore
    ):
        # Infos
        self._zone_id = zone_id
        self._properties = properties
        self._geolevel_id = geolevel_id
        self._zone_type_id = zone_type_id
        self._poi_id = poi_id
        self._parent_zone_id = parent_zone_id
        self._geometry = geometry
        # Org
        self._org = org

    # Getters and setters
    @property
    def zone_id(self): return self._zone_id
    @property
    def properties(self): return self._properties
    @property
    def geolevel_id(self): return self._geolevel_id
    @property
    def zone_type_id(self): return self._zone_type_id
    @property
    def poi_id(self): return self._poi_id
    @property
    def parent_zone_id(self): return self._parent_zone_id
    @property
    def geometry(self): return self._geometry
    @property
    def org(self): return self._org

    # Public Methods
    def getPoi(self)->Poi:
        query = f"SELECT * FROM ggo_poi WHERE poi_id = {self.poi_id}"
        gdf = gpd.read_postgis(query, self._org.engine, geom_col='geom')
        if len(gdf) > 0:
            data = gdf.iloc[0].to_dict()
            data.update({"org": self._org})
            return Poi(**data)
        else:
            raise Warning(f"Poi {self.poi_id} not found in Zone {self.zone_id}")
        
    def getParentZone(self) -> 'Zone':
        query = f"SELECT * FROM ggo_zone WHERE zone_id = {self.parent_zone_id}"
        df = pd.read_sql(query, self._org.engine)
        if len(df) > 0:
            data = df.iloc[0].to_dict()
            data.update({"org": self._org})
            return Zone(**data)
        else:
            raise Warning(f"Parent Zone {self.parent_zone_id} not found in Zone {self.zone_id}")
        
    def getChildrenZones(self)->list:
        query = f"SELECT * FROM ggo_zone WHERE parent_zone_id = {self.zone_id}"
        df = pd.read_sql(query, self._org.engine)
        zones = []
        for i in range(len(df)):
            data = df.iloc[i].to_dict()
            data.update({"org": self._org})
            zones.append(Zone(**data))
        return zones
    
    def getZoneGeounitsList(self) -> pd.DataFrame:
        query = f"SELECT * FROM ggo_zone_geounit WHERE zone_id = {self.zone_id}"
        return pd.read_sql(query, self._org.engine)
    
    def getZoneGeounitById(self, geounit_id:int) -> pd.DataFrame:
        query = f"SELECT * FROM ggo_zone_geounit WHERE zone_id = {self.zone_id} AND zone_geounit_id = {geounit_id}"
        df = pd.read_sql(query, self._org.engine)
        if len(df) > 0:
            data = df.iloc[0].to_dict()
            data.update({"org": self._org})
            return ZoneGeounit(**data)
        else:
            raise Warning(f"Geounit {geounit_id} not found in Zone {self.zone_id}")
        
    def getAllZoneGeounits(self) -> list:
        query = f"SELECT * FROM ggo_zone_geounit WHERE zone_id = {self.zone_id}"
        df = pd.read_sql(query, self._org.engine)
        zone_geounits = []
        for i in range(len(df)):
            data = df.iloc[i].to_dict()
            data.update({"org": self._org})
            zone_geounits.append(ZoneGeounit(**data))
        return zone_geounits
    
    def getGeolevel(self):
        return self._org.getGeolevelById(self.geolevel_id)
    
    def getGeoDataFrame(self)->gpd.GeoDataFrame:
        query = f"SELECT * FROM ggo_zone WHERE zone_id = {self.zone_id} AND geometry IS NOT NULL"
        gdf = gpd.read_postgis(query, self._org.engine, geom_col='geometry')
        if len(gdf) > 0:
            return gdf
        else:
            geolevel = self.getGeolevel()
            q = f"""
                SELECT
                    g.{geolevel.geounit_code} AS geounit_code,
                    z.properties AS zone_properties,
                    zg.properties AS zone_geounit_properties,
                    g.{geolevel.geom_field} AS geometry
                FROM ggo_zone z
                JOIN ggo_zone_geounit zg ON zg.zone_id = z.zone_id
                JOIN {geolevel.table_name} g ON g.{geolevel.geounit_code} = zg.geounit_code
                WHERE z.zone_id = {self.zone_id}
            """
            gdf = gpd.read_postgis(q, self._org.engine, geom_col=geolevel.geom_field)
            return gdf

    def add_to_model(self) -> int:
        if (self.zone_id is not None):
            # Update database
            geom_q = f"ST_GeomFromGeoJSON('{json.dumps(mapping(self.geometry))}')" if self.geometry is not None else 'NULL'
            query = f"""
            UPDATE ggo_zone SET
                properties = '{json.dumps(self.properties).replace("'", "''")}',
                geolevel_id = {self.geolevel_id if self.geolevel_id is not None else 'NULL'},
                zone_type_id = {self.zone_type_id},
                poi_id = {self.poi_id},
                parent_zone_id = {self.parent_zone_id if self.parent_zone_id is not None else 'NULL'},
                geometry = {geom_q}
            WHERE zone_id = {self.zone_id} RETURNING zone_id;
            """
            zone_id = self._org.query(query)[0][0]
            return zone_id
        # Add to database
        geom_q = f"ST_GeomFromGeoJSON('{json.dumps(mapping(self.geometry))}')" if self.geometry is not None else 'NULL'
        query = f"""
        INSERT INTO ggo_zone (
            properties,
            geolevel_id,
            zone_type_id,
            poi_id,
            parent_zone_id,
            geometry
        ) VALUES (
            '{json.dumps(self.properties).replace("'", "''")}',
            {self.geolevel_id if self.geolevel_id is not None else 'NULL'},
            {self.zone_type_id},
            {self.poi_id},
            {self.parent_zone_id if self.parent_zone_id is not None else 'NULL'},
            {geom_q}
        ) RETURNING zone_id;
        """
        zone_id = self._org.query(query)[0][0]
        return zone_id

    def plot(self, prop:str=None, plot_poi:bool=False)->'Axes':  # type: ignore
        # Get geolevel
        geolevel = self._org.getGeolevelById(self.geolevel_id)
        # Get list of geounits
        geounits = self.getZoneGeounitsList()
        geounits_code_list = geounits["geounit_code"].to_list()
        # Get geodata
        gdf = geolevel.getGeoDataset(geounits=geounits_code_list)
        # Plot
        if prop is not None:
            geounits["property_to_plot"] = geounits["properties"].apply(lambda x : x[prop])
            geounits = geounits[['geounit_code', 'property_to_plot']]
            gdf = gdf.merge(geounits, on="geounit_code")
            ax = gdf.plot(column='property_to_plot', legend=True)
            if plot_poi:
                poi = self.getPoi()
                x, y = poi.x, poi.y
                ax.scatter(x, y, color='red')
        else:
            ax = gdf.plot()
            if plot_poi:
                poi = self.getPoi()
                x, y = poi.x, poi.y
                ax.scatter(x, y, color='red')
        return ax


import pandas as pd
import geopandas as gpd
import json

from sqlalchemy import text

class Geolevel:
    # Constructor
    def __init__(
            self,
            geolevel_id:int,
            name:str,
            geounit_code:str,
            table_name:str,
            geom_field:str,
            geom_centroid_field:str,
            country:str,
            country_iso3:str,
            level:str,
            description:str,
            properties:str,
            sociodemo:dict,
            org:'Org' # type: ignore
    ):
        # Infos
        self._geolevel_id = geolevel_id
        self._name = name
        self._geounit_code = geounit_code
        self._table_name = table_name
        self._geom_field = geom_field
        self._geom_centroid_field = geom_centroid_field
        self._country = country
        self._country_iso3 = country_iso3
        self._level = level
        self._description = description
        self._properties = properties
        self._sociodemo = sociodemo
        # Engine
        self._org = org

    # Getters and setters
    @property
    def geolevel_id(self): return self._geolevel_id
    @property
    def name(self): return self._name
    @property
    def geounit_code(self): return self._geounit_code
    @property
    def table_name(self): return self._table_name
    @property
    def geom_field(self): return self._geom_field
    @property
    def geom_centroid_field(self): return self._geom_centroid_field
    @property
    def country(self): return self._country
    @property
    def country_iso3(self): return self._country_iso3
    @property
    def level(self): return self._level
    @property
    def description(self): return self._description
    @property
    def properties(self): return self._properties
    @property
    def sociodemo(self): return self._sociodemo
    @property
    def org(self): return self._org

    @name.setter
    def name(self, value): self._name = value
    @description.setter
    def description(self, value): self._description = value

    # Public Methods
    def getGeounitCodesList(self)->list:
        query = f"SELECT {self._geounit_code} FROM {self._table_name}"
        data = self._org.query(query)
        result = [row[0] for row in data]
        return result

    def getGeoDataset(self, all_data:bool=False, compute_centroid:bool=True, compute_centroid_crs:str="EPSG:2154", geounits:list=None)->gpd.GeoDataFrame:
        # Query
        query = f"SELECT * FROM {self._table_name}"
        if geounits is not None:
            str_geounits = ["'" + str(g) + "'" for g in geounits]
            query = f"{query} WHERE {self._geounit_code} IN ({', '.join([str(geounit) for geounit in str_geounits])})"
        # Get data from query
        gdf = gpd.read_postgis(query, self._org.engine, geom_col=self._geom_field)
        # Geounit_code column
        if self._geounit_code != "geounit_code":
            gdf["geounit_code"] = gdf[self._geounit_code]
            gdf = gdf.drop(columns=[self._geounit_code])
        gdf = gdf[["geounit_code"] + [col for col in gdf.columns if col != "geounit_code"]]
        # Centroid geometry
        if self._geom_centroid_field is None:
            if compute_centroid:
                gdf["geom_centroid"] = gdf[self._geom_field].to_crs(compute_centroid_crs).centroid.to_crs(gdf.crs)
            else:
                gdf["geom_centroid"] = None
        else:
            # Drop column
            gdf = gdf.drop(columns=[self._geom_centroid_field])
            query = f"SELECT {self._geounit_code}, {self._geom_centroid_field} FROM {self._table_name}"
            if geounits is not None:
                str_geounits = ["'" + str(g) + "'" for g in geounits]
                query = f"{query} WHERE {self._geounit_code} IN ({', '.join([geounit for geounit in str_geounits])})"
            # Get data from query
            gdf_centroid = gpd.read_postgis(query, self._org.engine, geom_col=self._geom_centroid_field)
            # Merge
            gdf = gdf.merge(gdf_centroid, left_on="geounit_code", right_on=self._geounit_code)
            # Rename column
            gdf = gdf.rename(columns={self._geom_centroid_field: "geom_centroid"})
        # Geometry in last
        if self._geom_field != "geom":
            gdf["geom"] = gdf[self._geom_field]
            gdf = gdf.drop(columns=[self._geom_field])
            gdf = gdf.set_geometry("geom")
        gdf = gdf[[col for col in gdf.columns if col != "geom"] + ["geom"]]
        # If not all data
        if not all_data:
            gdf = gdf[["geounit_code", "geom_centroid", "geom"]]
        # return df
        return gdf

    def getGeoCentroidDataset(self, geounits:list=None)->gpd.GeoDataFrame:
        # Query
        query = f"SELECT {self._geounit_code}, {self._geom_centroid_field} FROM {self._table_name}"
        if geounits is not None:
            str_geounits = ["'" + str(g) + "'" for g in geounits]
            query = f"{query} WHERE {self._geounit_code} IN ({', '.join([str(geounit) for geounit in str_geounits])})"
        # Get data from query
        gdf = gpd.read_postgis(query, self._org.engine, geom_col=self._geom_centroid_field)
        # Geounit_code column
        if self._geounit_code != "geounit_code":
            gdf["geounit_code"] = gdf[self._geounit_code]
            gdf = gdf.drop(columns=[self._geounit_code])
        gdf = gdf[["geounit_code"] + [col for col in gdf.columns if col != "geounit_code"]]
        # Geometry in last
        gdf = gdf.set_geometry(self._geom_centroid_field)
        # return df
        return gdf

    def getSocioDemoDataset(self, geounits:list=None)->pd.DataFrame:
        # Query
        query = self._sociodemo["query"]
        if geounits is not None:
            query = f"{query} WHERE {self._geounit_code} IN ({', '.join([str(geounit) for geounit in geounits])})"
        if query is not None:
            # Get data from query
            df = pd.read_sql(query, self._org.engine)
            # Geometry delete
            if "geometry" in df.columns:
                df = df.drop(columns=["geometry"], errors="ignore")
            else:
                raise Warning(f"No geometry column found in socio-demo query for geolevel {self._name}")
            # Geounit_code column in first
            if "geounit_code" in df.columns:
                df = df[["geounit_code"] + [col for col in df.columns if col != "geounit_code"]]
            else:
                raise Warning(f"No geounit_code column found in socio-demo query for geolevel {self._name}")
            # return df
            return df
        else:
            raise Warning(f"No socio-demo query found for geolevel {self._name}")
        
    def getGeoSocioDemoDataset(self, geounits:list=None)->gpd.GeoDataFrame:
        # Get geo dataset
        gdf = self.getGeoDataset(geounits=geounits)
        crs = gdf.crs
        gdf = gdf.drop(columns=["geom_centroid"])
        # Get socio demo dataset
        df = self.getSocioDemoDataset(geounits=geounits)
        # Merge
        gdf = pd.merge(df, gdf, on="geounit_code")
        # GeoDataFrame
        gdf = gpd.GeoDataFrame(gdf, geometry="geom", crs=crs)
        # return df
        return gdf
    
    def getJson(self)->dict:
        return {
            "geolevel_id": self._geolevel_id,
            "name": self._name,
            "geounit_code": self._geounit_code,
            "table_name": self._table_name,
            "geom_field": self._geom_field,
            "geom_centroid_field": self._geom_centroid_field,
            "country": self._country,
            "country_iso3": self._country_iso3,
            "level": self._level,
            "description": self._description,
            "properties": self._properties,
            "sociodemo": self._sociodemo
        }
    
    def add_to_model(self) -> int:
        # Add to database
        query = f"""
        INSERT INTO ggo_geolevel (
            name,
            geounit_code,
            table_name,
            geom_field,
            geom_centroid_field,
            country,
            country_iso3,
            level,
            description,
            properties,
            sociodemo
        )
        VALUES (
            '{self._name.replace("'", "''")}',
            '{self._geounit_code}',
            '{self._table_name}',
            '{self._geom_field}',
            {"'" + self._geom_centroid_field + "'" if self._geom_centroid_field is not None else 'NULL'},
            '{self._country.replace("'", "''")}',
            '{self._country_iso3}',
            '{self._level.replace("'", "''")}',
            '{self._description.replace("'", "''")}',
            '{json.dumps(self._properties).replace("'", "''")}',
            '{json.dumps(self._sociodemo).replace("'", "''")}'
        )
        RETURNING geolevel_id;
        """
        geolevel_id = self._org.query(query)[0][0]
        # Return
        return geolevel_id
    
    def number_of_geounits(self)->int:
        query = text(f"SELECT COUNT(*) FROM {self._table_name}")
        with self._org.engine.connect() as conn:
            result = conn.execute(query)
            return result.scalar()
        
    def update(self) -> 'Geolevel':
        query = f"""
        UPDATE ggo_geolevel
        SET
            name = '{self._name.replace("'", "''")}',
            geounit_code = '{self._geounit_code}',
            table_name = '{self._table_name}',
            geom_field = '{self._geom_field}',
            geom_centroid_field = {"'" + self._geom_centroid_field + "'" if self._geom_centroid_field is not None else 'NULL'},
            country = '{self._country.replace("'", "''")}',
            country_iso3 = '{self._country_iso3}',
            level = '{self._level.replace("'", "''")}',
            description = '{self._description.replace("'", "''")}',
            properties = '{json.dumps(self._properties).replace("'", "''")}',
            sociodemo = '{json.dumps(self._sociodemo).replace("'", "''")}'
        WHERE geolevel_id = {self._geolevel_id}
        """
        self._org.query(query)
        return self._org.getGeolevelById(self._geolevel_id)
    
    def delete(self)-> bool:
        query = f"DELETE FROM ggo_geolevel WHERE geolevel_id = {self._geolevel_id}"
        self._org.query(query)
        self._geolevel_id = None
        return True


import pandas as pd
from sqlalchemy import text

class ZoneGeounit:
    # Constructor
    def __init__(
        self,
        zone_geounit_id:int,
        geounit_code:str,
        properties:dict,
        zone_id:int,
        org:'Org' # type: ignore
    ):
        # Infos
        self._zone_geounit_id = zone_geounit_id
        self._geounit_code = geounit_code
        self._properties = properties
        self._zone_id = zone_id
        # Engine
        self._org = org

    # Getters and setters
    @property
    def zone_geounit_id(self): return self._zone_geounit_id
    @property
    def geounit_code(self): return self._geounit_code
    @property
    def properties(self): return self._properties
    @property
    def zone_id(self): return self._zone_id
    @property
    def org(self): return self._org

    # Public Methods
    def getZone(self)->'Zone': # type: ignore
        query = text(f"SELECT zone_type_id FROM ggo_zone WHERE zone_id = {self.zone_id}")
        zone_type_id = self.org.query(query).iloc[0]["zone_type_id"]
        return self.org.getZoneTypeById(zone_type_id).getZoneById(self.zone_id)
    



import pandas as pd

class Poi:
    # Constructor
    def __init__(
            self,
            poi_id:int,
            id:str,
            address:str,
            postal_code:str,
            city:str,
            name:str,
            geounit_code:str,
            geocoder_accuracy:str,
            created_at:str,
            last_updated_at:str,
            last_updated_by:str,
            open_date:str,
            geom:str,
            properties:dict,
            network_id:int,
            logo_id:int,
            org:'Org' # type: ignore
    ):
        # Infos
        self._poi_id = poi_id
        self._id = id
        self._address = address
        self._postal_code = postal_code
        self._city = city
        self._name = name
        self._geounit_code = geounit_code
        self._geocoder_accuracy = geocoder_accuracy
        self._created_at = created_at
        self._last_updated_at = last_updated_at
        self._last_updated_by = last_updated_by
        self._open_date = open_date
        self._geom = geom
        self._properties = properties
        self._network_id = network_id
        self._logo_id = logo_id
        # Engine
        self._org = org

    # Getters and setters
    @property
    def poi_id(self): return self._poi_id
    @property
    def id(self): return self._id
    @property
    def address(self): return self._address
    @property
    def postal_code(self): return self._postal_code
    @property
    def city(self): return self._city
    @property
    def name(self): return self._name
    @property
    def geounit_code(self): return self._geounit_code
    @property
    def geocoder_accuracy(self): return self._geocoder_accuracy
    @property
    def created_at(self): return self._created_at
    @property
    def last_updated_at(self): return self._last_updated_at
    @property
    def last_updated_by(self): return self._last_updated_by
    @property
    def open_date(self): return self._open_date
    @property
    def geom(self): return self._geom
    @property
    def x(self): return float(self._geom.x)
    @property
    def y(self): return float(self._geom.y)
    @property
    def properties(self): return self._properties
    @property
    def network_id(self): return self._network_id
    @property
    def logo_id(self): return self._logo_id
    @property
    def org(self): return self._org

    # Magic Methods
    def __str__(self):
        return self.name + " (" + self.id + ")"
    



# IMPORTS
from sqlalchemy import create_engine, Engine, text
import numpy as np
import pandas as pd
import geopandas as gpd

from galigeopy.model.network import Network
from galigeopy.model.poi import Poi
from galigeopy.model.zone_type import ZoneType
from galigeopy.model.geolevel import Geolevel
from galigeopy.model.geolevel_data_type import GeolevelDataType
from galigeopy.model.distancier_session import DistancierSession

class Org:
    # Constructor
    def __init__(self, user, password, host="127.0.0.1", port=5432, db="galigeo"):
        # Infos
        self._user = user
        self._password = password
        self._host = host
        self._port = port
        self._db = db
        # Engine
        self._is_valid = False
        self._engine = self._get_engine()
        self._is_valid = self._check_validity()
        self._default_schema = self._get_default_schema()
    
    # Getters and setters
    @property
    def engine(self): return self._engine
    @property
    def is_valid(self): return self._is_valid
    @property
    def default_schema(self): return self._default_schema
    @property
    def name(self): return self._user.strip()

    # Private Methods
    def _get_engine(self) -> Engine:
        return create_engine(f'postgresql://{self._user}:{self._password}@{self._host}:{self._port}/{self._db}')

    def _get_default_schema(self)->str:
        query = "SELECT current_schema()"
        return self.query(query)[0][0]

    def _check_validity(self)->bool:
        # Check connection
        try:
            self._engine.connect()
            return True
        except Exception as e:
            return False
    
    # Public Methods
    def query_df(self, query:str, dtype:dict={})->pd.DataFrame:
        df = pd.read_sql_query(query, self._engine, dtype=dtype)
        return df

    def query(self, query:str)->list:
        with self._engine.connect() as conn:
            result = conn.execute(text(query))
            conn.commit()
            if result.returns_rows:
                return result.fetchall()
            else:
                return []

    def getNetworksList(self)->pd.DataFrame:
        # Query
        query = "SELECT * FROM ggo_network"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # return df
        return df
    
    def getNetworkById(self, id:int)->Network:
        # Query
        query = f"SELECT * FROM ggo_network WHERE network_id = {str(id)}"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # Data
        if len(df) > 0:
            data = df.iloc[0].to_dict()
            data.update({"org": self})
            return Network(**data)
        else:
            raise Warning(f"Network with id {id} not found")
    
    def getNetworkByName(self, name:str)->Network:
        # Query
        query = f"SELECT * FROM ggo_network WHERE name = '{name}'"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # Data
        if len(df) > 0:
            data = df.iloc[0].to_dict()
            data.update({"org": self})
            return Network(**data)
        else:
            raise Warning(f"Network with name {name} not found")
    
    def getAllNetworks(self)->list:
        # Query
        query = "SELECT * FROM ggo_network"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # Data
        networks = []
        for i in range(len(df)):
            data = df.iloc[i].to_dict()
            data.update({"org": self})
            networks.append(Network(**data))
        return networks
    
    def getPoisByCode(self, code:str, networks_ids:list=[])->list:
        # Query
        query = f"SELECT * FROM ggo_poi WHERE id = '{code}'" if len(networks_ids) == 0 else f"SELECT * FROM ggo_poi WHERE id = '{code}' AND network_id IN ({','.join([str(i) for i in networks_ids])})"
        # Get data from query
        gdf = gpd.read_postgis(query, self._engine, geom_col="geom")
        # Data
        pois = []
        for i in range(len(gdf)):
            data = gdf.iloc[i].to_dict()
            data.update({"org": self})
            pois.append(Poi(**data))
        return pois
    
    def getZoneTypesList(self)->pd.DataFrame:
        # Query
        query = "SELECT * FROM ggo_zone_type"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # return df
        return df
    
    def getZoneTypeById(self, id:int)->ZoneType:
        # Query
        query = f"SELECT * FROM ggo_zone_type WHERE zone_type_id = {str(id)}"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # Data
        if len(df) > 0:
            data = df.iloc[0].to_dict()
            data.update({"org": self})
            return ZoneType(**data)
        else:
            raise Warning(f"ZoneType with id {id} not found")
        
    def getZoneTypeByName(self, name:str)->ZoneType:
        # Query
        name_sql = name.replace("'", "''")
        query = f"SELECT * FROM ggo_zone_type WHERE name = '{name_sql}'"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # Data
        if len(df) > 0:
            data = df.iloc[0].to_dict()
            data.update({"org": self})
            return ZoneType(**data)
        else:
            raise Warning(f"ZoneType with name {name} not found")
        
    def getAllZoneTypes(self)->list:
        # Query
        query = "SELECT * FROM ggo_zone_type"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # Data
        zone_types = []
        for i in range(len(df)):
            data = df.iloc[i].to_dict()
            data.update({"org": self})
            zone_types.append(ZoneType(**data))
        return zone_types
    
    def getGeolevelsList(self)->pd.DataFrame:
        # Query
        query = "SELECT * FROM ggo_geolevel"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # return df
        return df
    
    def getGeolevelById(self, id:int)->Geolevel:
        # Query
        query = f"SELECT * FROM ggo_geolevel WHERE geolevel_id = {str(id)}"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # Data
        if len(df) > 0:
            data = df.iloc[0].to_dict()
            data.update({"org": self})
            return Geolevel(**data)
        else:
            raise Warning(f"Geolevel with id {id} not found")

    def getGeolevelByName(self, name:str)->Geolevel:
        # Query
        query = f"SELECT * FROM ggo_geolevel WHERE name = '{name}'"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # Data
        if len(df) > 0:
            data = df.iloc[0].to_dict()
            data.update({"org": self})
            return Geolevel(**data)
        else:
            raise Warning(f"Geolevel with name {name} not found")
        
    def getAllGeolevels(self)->list:
        # Query
        query = "SELECT * FROM ggo_geolevel"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # Data
        geolevels = []
        for i in range(len(df)):
            data = df.iloc[i].to_dict()
            data.update({"org": self})
            geolevels.append(Geolevel(**data))
        return geolevels
    
    def getGeolevelDataTypesList(self)->pd.DataFrame:
        # Query
        query = "SELECT * FROM ggo_geoleveldata_type"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # return df
        return df
    
    def getGeolevelDataTypeById(self, id:int)->GeolevelDataType:
        # Query
        query = f"SELECT * FROM ggo_geoleveldata_type WHERE custdata_type_id = {str(id)}"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # Data
        if len(df) > 0:
            data = df.iloc[0].to_dict()
            data.update({"org": self})
            return GeolevelDataType(**data)
        else:
            raise Warning(f"GeolevelDataType with id {id} not found")
        
    def getGeolevelDataTypeByName(self, name:str)->GeolevelDataType:
        # Query
        query = f"SELECT * FROM ggo_geoleveldata_type WHERE name = '{name}'"
        # Get data from query
        df = pd.read_sql(query, self._engine)
        # Data
        if len(df) > 0:
            data = df.iloc[0].to_dict()
            data.update({"org": self})
            return GeolevelDataType(**data)
        else:
            raise Warning(f"GeolevelDataType with name {name} not found")
        
    def getDistancierSessionList(self)->pd.DataFrame:
        # Query
        q = f"SELECT * FROM ggo_distancier_session"
        df = self.query_df(q)
        return df
    
    def getDistancierSessionById(self, session_id:int)->DistancierSession:
        # Query
        q = f"SELECT * FROM ggo_distancier_session WHERE session_id = {session_id}"
        df = self._org.query_df(q)
        if len(df) != 1:
            raise Warning(f"DistancierSession with id {session_id} doesn't exist")
        data = df.iloc[0].to_dict()
        data.update({"org": self})
        return DistancierSession(**data)
    
    def getDistancierSessionByName(self, name:str)->DistancierSession:
        # Query
        q = f"SELECT * FROM ggo_distancier_session WHERE name = '{name}'"
        df = self._org.query_df(q)
        if len(df) != 1:
            raise Warning(f"DistancierSession with name {name} doesn't exist")
        data = df.iloc[0].to_dict()
        data.update({"org": self})
        return DistancierSession(**data)
    
    def getAllDistancierSessions(self)->list:
        # query
        q = "SELECT * FROM ggo_distancier_session"
        df = self.query_df(q)
        # Data
        sessions = []
        for i in range(len(df)):
            data = df.iloc[i].to_dict()
            data.update({"org": self})
            s = DistancierSession(**data)
            sessions.append(s)
        return sessions


    



from pathlib import Path

import setuptools

VERSION = "0.0.44"  # PEP-440

NAME = "galigeopy"

INSTALL_REQUIRES = [
]


setuptools.setup(
    name=NAME,
    version=VERSION,
    description="Galigeo Python SDK",
    url="https://github.com/JPTGGO/galigeopy",
    project_urls={
        "Source Code": "https://github.com/JPTGGO/galigeopy",
    },
    author="Jules Pierrat",
    author_email="jpierrat@galigeo.com",
    license="Apache License 2.0",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3.10",
    ],
    # Snowpark requires Python 3.8
    python_requires=">=3.8",
    # Requirements
    install_requires=INSTALL_REQUIRES,
    packages=setuptools.find_packages(exclude=["tests"]),
    long_description=Path("README.md").read_text(),
    long_description_content_type="text/markdown",
)


# Haversine distance formula
from math import asin, cos, radians, sin, sqrt, pi
import numpy as np

def haversine_distance(lat1, lon1, lat2, lon2, rayon=6371000)->list|float:
    """
    Calculate the great circle distance between two points
    on the earth (specified in decimal degrees)
    """
    # convert decimal degrees to radians
    lat1 = np.array(lat1) * pi / 180
    lon1 = np.array(lon1) * pi / 180
    lat2 = np.array(lat2) * pi / 180
    lon2 = np.array(lon2) * pi / 180
    # haversine formula
    dlat = lat2 - lat1
    dlon = lon2 - lon1
    a = np.sin(dlat/2)**2 + np.cos(lat1) * np.cos(lat2) * np.sin(dlon/2)**2
    c = 2 * np.asin(np.sqrt(a))
    km = rayon * c
    return km